#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>


#define PWD_BUFFER_SIZE 256

#ifndef TCSASOFT
#define TCSASOFT 0
#endif


char * getpass (const char *prompt)
{
  FILE *in, *out;
  struct termios s, t;
  int tty_changed;
  static char buf[PWD_BUFFER_SIZE];
  int nread;

  /* Try to write to and read from the terminal if we can.
 *      If we can't open the terminal, use stderr and stdin.  */

  out = in = fopen ("/dev/tty", "r+");
  if (in == NULL)
    {
      in = stdin;
      out = stderr;
    }
  else
    {
      /* Disable buffering for read/write FILE to prevent problems with
 *        * fseek and buffering for read/write auto-transitioning. */
      setvbuf(in, NULL, _IONBF, 0);
    }

  /* Turn echoing off if it is on now.  */

  tty_changed = 0;
  if (tcgetattr (fileno (in), &t) == 0)
    {
      /* Save the old one. */
      s = t;
      /* Tricky, tricky. */
      t.c_lflag &= ~(ECHO|ISIG);
      tty_changed = (tcsetattr (fileno (in), TCSAFLUSH|TCSASOFT, &t) == 0);
    }

  /* Write the prompt.  */
  fputs(prompt, out);
  fflush(out);

  /* Read the password.  */
  if (!fgets (buf, sizeof(buf), in))
    buf[0] = '\0';
  nread = strlen(buf);
  if (nread > 0 && buf[nread - 1] == '\n')
      /* Remove the newline.  */
      buf[nread - 1] = '\0';

  if (tty_changed)
    {
      /* Write the newline that was not echoed.  */
      putc('\n', out);
      /* Restore the original setting.  */
      (void) tcsetattr (fileno (in), TCSAFLUSH|TCSASOFT, &s);
    }

  if (in != stdin)
    /* We opened the terminal; now close it.  */
    fclose (in);

  return buf;
}


